@@ -24,23 +24,36 @@
   @NOTATION@
 */
 
+/*
+ * NB. this may look like a GtkBin, but it contains
+ * a BonoboDockItemGrip in addition to it's child,
+ * stranger things have been done in the name of
+ * bin-compat.
+ */
+
+#include <config.h>
+
 #include <gdk/gdkx.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtkwindow.h>
 
-#include <bonobo/bonobo-dock-item.h>
+#include <libgnome/gnome-macros.h>
 #include <bonobo/bonobo-i18n.h>
+#include <bonobo/bonobo-dock-band.h>
+#include <bonobo/bonobo-dock-item.h>
+#include <bonobo/bonobo-ui-private.h>
+#include <bonobo/bonobo-ui-marshal.h>
+#include <bonobo/bonobo-dock-item-grip.h>
 
-struct _BonoboDockItemPrivate
-{
-	int dummy;
-	/* Nothing right now, needs to get filled with the private things */
-	/* XXX: When stuff is added, uncomment the allocation in the
-	 * bonobo_dock_item_init function! */
+struct _BonoboDockItemPrivate {
+	GtkWidget *grip;
 };
 
+GNOME_CLASS_BOILERPLATE (BonoboDockItem, bonobo_dock_item,
+			 GtkBin, GTK_TYPE_BIN);
 
 enum {
   PROP_0,
@@ -61,49 +74,52 @@ enum {
   LAST_SIGNAL
 };
 
+/* this function is not public, but should be exported */
+void        bonobo_dock_item_set_behavior   (BonoboDockItem         *dock_item,
+                                             BonoboDockItemBehavior  behavior);
+
 
 static guint     get_preferred_width   (BonoboDockItem *item);
 static guint     get_preferred_height  (BonoboDockItem *item);
 
-static void bonobo_dock_item_class_init     (BonoboDockItemClass *klass);
-static void bonobo_dock_item_init           (BonoboDockItem      *dock_item);
 static void bonobo_dock_item_set_property   (GObject            *object,
-					    guint               param_id,
-					    const GValue       *value,
-					    GParamSpec         *pspec);
+					     guint               param_id,
+					     const GValue       *value,
+					     GParamSpec         *pspec);
 static void bonobo_dock_item_get_property   (GObject            *object,
-					    guint               param_id,
-					    GValue             *value,
-					    GParamSpec         *pspec);
-static void bonobo_dock_item_destroy        (GtkObject         *object);
+					     guint               param_id,
+					     GValue             *value,
+					     GParamSpec         *pspec);
 static void bonobo_dock_item_finalize       (GObject           *object);
 static void bonobo_dock_item_map            (GtkWidget         *widget);
 static void bonobo_dock_item_unmap          (GtkWidget         *widget);
 static void bonobo_dock_item_realize        (GtkWidget         *widget);
 static void bonobo_dock_item_unrealize      (GtkWidget         *widget);
 static void bonobo_dock_item_style_set      (GtkWidget         *widget,
-                                            GtkStyle          *previous_style);
+					     GtkStyle          *previous_style);
 static void bonobo_dock_item_size_request   (GtkWidget         *widget,
-                                            GtkRequisition    *requisition);
+					     GtkRequisition    *requisition);
 static void bonobo_dock_item_size_allocate  (GtkWidget         *widget,
-                                            GtkAllocation     *real_allocation);
+					     GtkAllocation     *real_allocation);
 static void bonobo_dock_item_add            (GtkContainer      *container,
-                                            GtkWidget         *widget);
+					     GtkWidget         *widget);
 static void bonobo_dock_item_remove         (GtkContainer      *container,
-                                            GtkWidget         *widget);
+					     GtkWidget         *widget);
+static void bonobo_dock_item_forall         (GtkContainer     *container,
+					     gboolean          include_internals,
+					     GtkCallback       callback,
+					     gpointer          callback_data);
 static void bonobo_dock_item_paint          (GtkWidget         *widget,
-                                            GdkEventExpose    *event);
+					     GdkEventExpose    *event);
 static gint bonobo_dock_item_expose         (GtkWidget         *widget,
-                                            GdkEventExpose    *event);
+					     GdkEventExpose    *event);
 static gint bonobo_dock_item_button_changed (GtkWidget         *widget,
-                                            GdkEventButton    *event);
+					     GdkEventButton    *event);
 static gint bonobo_dock_item_motion         (GtkWidget         *widget,
-                                            GdkEventMotion    *event);
+					     GdkEventMotion    *event);
 static gint bonobo_dock_item_delete_event   (GtkWidget         *widget,
-                                            GdkEventAny       *event);
+					     GdkEventAny       *event);
 
-
-static GtkBinClass *parent_class;
 static guint        dock_item_signals[LAST_SIGNAL] = { 0 };
 
 
@@ -178,30 +194,78 @@ get_preferred_height (BonoboDockItem *dock_item)
   return preferred_height;
 }
 
-
-GtkType
-bonobo_dock_item_get_type (void)
+static gboolean
+bonobo_gtk_container_focus_move (GtkContainer     *container,
+				 GList            *children,
+				 GtkDirectionType  direction)
 {
-  static GtkType dock_item_type = 0;
+  GtkWidget *focus_child;
+  GtkWidget *child;
 
-  if (!dock_item_type)
+  focus_child = container->focus_child;
+
+  while (children)
     {
-      GtkTypeInfo dock_item_info =
-      {
-	"BonoboDockItem",
-	sizeof (BonoboDockItem),
-	sizeof (BonoboDockItemClass),
-	(GtkClassInitFunc) bonobo_dock_item_class_init,
-	(GtkObjectInitFunc) bonobo_dock_item_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
-      };
+      child = children->data;
+      children = children->next;
 
-      dock_item_type = gtk_type_unique (gtk_bin_get_type (), &dock_item_info);
+      if (!child)
+	continue;
+      
+      if (focus_child)
+        {
+          if (focus_child == child)
+            {
+              focus_child = NULL;
+
+		if (gtk_widget_child_focus (child, direction))
+		  return TRUE;
+            }
+        }
+      else if (GTK_WIDGET_DRAWABLE (child) &&
+               gtk_widget_is_ancestor (child, GTK_WIDGET (container)))
+        {
+          if (gtk_widget_child_focus (child, direction))
+            return TRUE;
+        }
     }
 
-  return dock_item_type;
+  return FALSE;
+}
+
+/*
+ * FIXME: this is a hack because in un-docked toolbars
+ * bad things happen if we traverse up the GdkWindow
+ * hierarchy doing dumb things like GtkContainer does.
+ */
+gboolean
+bonobo_widget_clobber_focus (GtkWidget        *widget,
+			     GtkDirectionType  direction)
+{
+  gboolean has_focus_chain;
+  GList *focus_chain = NULL;
+  GList *children = NULL;
+  gboolean return_val = FALSE;
+  GtkContainer *container;
+
+  container = GTK_CONTAINER (widget);
+
+  has_focus_chain = gtk_container_get_focus_chain (
+	  container, &focus_chain);
+
+  if (has_focus_chain)
+    children = g_list_copy (focus_chain);
+  else
+    children = gtk_container_get_children (container);
+
+  if (direction == GTK_DIR_TAB_BACKWARD || direction == GTK_DIR_LEFT)
+    children = g_list_reverse (children);
+
+  return_val = bonobo_gtk_container_focus_move (container, children, direction);
+
+  g_list_free (children);
+
+  return return_val;
 }
 
 static void
@@ -217,94 +281,100 @@ bonobo_dock_item_class_init (BonoboDockItemClass *class)
   widget_class = (GtkWidgetClass *) class;
   container_class = (GtkContainerClass *) class;
 
-  parent_class = gtk_type_class (gtk_bin_get_type ());
-
   gobject_class->set_property = bonobo_dock_item_set_property;
   gobject_class->get_property = bonobo_dock_item_get_property;
   
-  g_object_class_install_property (gobject_class,
-				   PROP_SHADOW,
-				   g_param_spec_enum ("shadow",
-						      _("Shadow type"),
-						      _("Shadow type"),
-						      GTK_TYPE_SHADOW_TYPE,
-						      GTK_SHADOW_OUT,
-						      (G_PARAM_READABLE |
-						       G_PARAM_WRITABLE)));
-  g_object_class_install_property (gobject_class,
-				   PROP_ORIENTATION,
-				   g_param_spec_enum ("orientation",
-						      _("Orientation"),
-						      _("Orientation"),
-						      GTK_TYPE_ORIENTATION,
-						      GTK_ORIENTATION_HORIZONTAL,
-						      (G_PARAM_READABLE |
-						       G_PARAM_WRITABLE)));
-  g_object_class_install_property (gobject_class,
-				   PROP_PREFERRED_WIDTH,
-				   g_param_spec_uint ("preferred_width",
-						      _("Preferred width"),
-						      _("Preferred width"),
-						      0, G_MAXINT, 0,
-						      (G_PARAM_READABLE |
-						       G_PARAM_WRITABLE)));
-  g_object_class_install_property (gobject_class,
-				   PROP_PREFERRED_HEIGHT,
-				   g_param_spec_uint ("preferred_height",
-						      _("Preferred height"),
-						      _("Preferred height"),
-						      0, G_MAXINT, 0,
-						      (G_PARAM_READABLE |
-						       G_PARAM_WRITABLE)));
+  g_object_class_install_property (
+	  gobject_class,
+	  PROP_SHADOW,
+	  g_param_spec_enum ("shadow",
+			     _("Shadow type"),
+			     _("Shadow type"),
+			     GTK_TYPE_SHADOW_TYPE,
+			     GTK_SHADOW_OUT,
+			     (G_PARAM_READABLE |
+			      G_PARAM_WRITABLE)));
+
+  g_object_class_install_property (
+	  gobject_class,
+	  PROP_ORIENTATION,
+	  g_param_spec_enum ("orientation",
+			     _("Orientation"),
+			     _("Orientation"),
+			     GTK_TYPE_ORIENTATION,
+			     GTK_ORIENTATION_HORIZONTAL,
+			     (G_PARAM_READABLE |
+			      G_PARAM_WRITABLE)));
+
+  g_object_class_install_property (
+	  gobject_class,
+	  PROP_PREFERRED_WIDTH,
+	  g_param_spec_uint ("preferred_width",
+			     _("Preferred width"),
+			     _("Preferred width"),
+			     0, G_MAXINT, 0,
+			     G_PARAM_READABLE));
+
+  g_object_class_install_property (
+	  gobject_class,
+	  PROP_PREFERRED_HEIGHT,
+	  g_param_spec_uint ("preferred_height",
+			     _("Preferred height"),
+			     _("Preferred height"),
+			     0, G_MAXINT, 0,
+			     G_PARAM_READABLE));
 
   dock_item_signals[DOCK_DRAG_BEGIN] =
-    gtk_signal_new ("dock_drag_begin",
-                    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (BonoboDockItemClass, dock_drag_begin),
-                    gtk_marshal_NONE__NONE,
-                    GTK_TYPE_NONE, 0);
+	  g_signal_new ("dock_drag_begin",
+			G_TYPE_FROM_CLASS (object_class),
+			G_SIGNAL_RUN_LAST,
+			G_STRUCT_OFFSET (BonoboDockItemClass,
+					 dock_drag_begin),
+			NULL, NULL,
+			g_cclosure_marshal_VOID__VOID,
+			G_TYPE_NONE, 0);
 
   dock_item_signals[DOCK_DRAG_MOTION] =
-    gtk_signal_new ("dock_drag_motion",
-                    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (BonoboDockItemClass, dock_drag_motion),
-                    gtk_marshal_NONE__INT_INT,
-                    GTK_TYPE_NONE, 2,
-                    GTK_TYPE_INT,
-                    GTK_TYPE_INT);
+	  g_signal_new ("dock_drag_motion",
+			G_TYPE_FROM_CLASS (object_class),
+			G_SIGNAL_RUN_LAST,
+			G_STRUCT_OFFSET (BonoboDockItemClass, dock_drag_motion),
+			NULL, NULL,
+			bonobo_ui_marshal_VOID__INT_INT,
+			G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 
   dock_item_signals[DOCK_DRAG_END] =
-    gtk_signal_new ("dock_drag_end",
-                    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (BonoboDockItemClass, dock_drag_end),
-                    gtk_marshal_NONE__NONE,
-                    GTK_TYPE_NONE, 0);
+	  g_signal_new ("dock_drag_end",
+			G_TYPE_FROM_CLASS (object_class),
+			G_SIGNAL_RUN_LAST,
+			G_STRUCT_OFFSET (BonoboDockItemClass, dock_drag_end),
+			NULL, NULL,
+			g_cclosure_marshal_VOID__VOID,
+			G_TYPE_NONE, 0);
 
   dock_item_signals[DOCK_DETACH] =
-    gtk_signal_new ("dock_detach",
-                    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (BonoboDockItemClass, dock_detach),
-                    gtk_marshal_NONE__NONE,
-                    GTK_TYPE_NONE, 0);
+	  g_signal_new ("dock_detach",
+			G_TYPE_FROM_CLASS (object_class),
+			G_SIGNAL_RUN_LAST,
+			G_STRUCT_OFFSET (BonoboDockItemClass, dock_detach),
+			NULL, NULL,
+			g_cclosure_marshal_VOID__VOID,
+			G_TYPE_NONE, 0);
 
   dock_item_signals[ORIENTATION_CHANGED] =
-    gtk_signal_new ("orientation_changed",
-		    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (BonoboDockItemClass, orientation_changed),
-		    gtk_marshal_VOID__ENUM,
-		    GTK_TYPE_NONE, 0, GTK_TYPE_ENUM);
-
+	  g_signal_new ("orientation_changed",
+			G_TYPE_FROM_CLASS (object_class),
+			G_SIGNAL_RUN_LAST,
+			G_STRUCT_OFFSET (BonoboDockItemClass, orientation_changed),
+			NULL, NULL,
+			g_cclosure_marshal_VOID__ENUM,
+			G_TYPE_NONE, 1, GTK_TYPE_ORIENTATION);
   
-  object_class->destroy = bonobo_dock_item_destroy;
   gobject_class->finalize = bonobo_dock_item_finalize;
 
   widget_class->map = bonobo_dock_item_map;
   widget_class->unmap = bonobo_dock_item_unmap;
+  widget_class->focus = bonobo_widget_clobber_focus;
   widget_class->realize = bonobo_dock_item_realize;
   widget_class->unrealize = bonobo_dock_item_unrealize;
   widget_class->style_set = bonobo_dock_item_style_set;
@@ -318,17 +388,19 @@ bonobo_dock_item_class_init (BonoboDockItemClass *class)
 
   container_class->add = bonobo_dock_item_add;
   container_class->remove = bonobo_dock_item_remove;
+  container_class->forall = bonobo_dock_item_forall;
 }
 
 static void
-bonobo_dock_item_init (BonoboDockItem *dock_item)
+bonobo_dock_item_instance_init (BonoboDockItem *dock_item)
 {
   GTK_WIDGET_UNSET_FLAGS (dock_item, GTK_NO_WINDOW);
 
-  dock_item->_priv = NULL;
-  /* XXX: when there is some private stuff enable this
-  dock_item->_priv = g_new0(BonoboDockItemPrivate, 1);
-  */
+  dock_item->_priv = g_new (BonoboDockItemPrivate, 1);
+
+  dock_item->_priv->grip = bonobo_dock_item_grip_new (dock_item);
+  gtk_widget_set_parent (dock_item->_priv->grip, GTK_WIDGET (dock_item));
+  gtk_widget_show (dock_item->_priv->grip);
 
   dock_item->bin_window = NULL;
   dock_item->float_window = NULL;
@@ -408,25 +480,6 @@ bonobo_dock_item_get_property (GObject            *object,
     }
 }
 
-static void
-bonobo_dock_item_destroy (GtkObject *object)
-{
-  BonoboDockItem *di;
-
-  /* remember, destroy can be run multiple times! */
-
-  g_return_if_fail (object != NULL);
-  g_return_if_fail (BONOBO_IS_DOCK_ITEM (object));
-
-  di = BONOBO_DOCK_ITEM (object);
-
-  g_free (di->name);
-  di->name = NULL;
-
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
 static void
 bonobo_dock_item_finalize (GObject *object)
 {
@@ -437,12 +490,13 @@ bonobo_dock_item_finalize (GObject *object)
 
   di = BONOBO_DOCK_ITEM (object);
 
-  /* Free the private structure */
+  g_free (di->name);
+  di->name = NULL;
+
   g_free (di->_priv);
   di->_priv = NULL;
 
-  if (G_OBJECT_CLASS (parent_class)->finalize)
-    (* G_OBJECT_CLASS (parent_class)->finalize) (object);
+  GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static void
@@ -470,6 +524,11 @@ bonobo_dock_item_map (GtkWidget *widget)
       && GTK_WIDGET_VISIBLE (bin->child)
       && !GTK_WIDGET_MAPPED (bin->child))
     gtk_widget_map (bin->child);
+
+  if (di->_priv->grip
+      && GTK_WIDGET_VISIBLE (di->_priv->grip)
+      && !GTK_WIDGET_MAPPED (di->_priv->grip))
+    gtk_widget_map (di->_priv->grip);
 }
 
 static void
@@ -490,12 +549,16 @@ bonobo_dock_item_unmap (GtkWidget *widget)
       gdk_window_hide (di->float_window);
       di->float_window_mapped = FALSE;
     }
+
+  if (di->_priv->grip)
+    gtk_widget_unmap (di->_priv->grip);
 }
 
 static void
 bonobo_dock_item_realize (GtkWidget *widget)
 {
   GdkWindowAttr attributes;
+  GdkWindow *root_window;
   gint attributes_mask;
   BonoboDockItem *di;
 
@@ -530,12 +593,24 @@ bonobo_dock_item_realize (GtkWidget *widget)
 			    GDK_BUTTON1_MOTION_MASK |
 			    GDK_POINTER_MOTION_HINT_MASK |
 			    GDK_BUTTON_PRESS_MASK |
-			    GDK_BUTTON_RELEASE_MASK);
+			    GDK_BUTTON_RELEASE_MASK |
+	                    GDK_KEY_PRESS_MASK);
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
   di->bin_window = gdk_window_new (widget->window, &attributes, attributes_mask);
   gdk_window_set_user_data (di->bin_window, widget);
+
   if (GTK_BIN (di)->child)
     gtk_widget_set_parent_window (GTK_BIN (di)->child, di->bin_window);
+
+  if (di->_priv->grip)
+    gtk_widget_set_parent_window (di->_priv->grip, di->bin_window);
+
+#ifdef HAVE_GTK_MULTIHEAD
+  root_window = gdk_screen_get_root_window (
+			gdk_drawable_get_screen (GDK_DRAWABLE (widget->window)));
+#else
+  root_window = gdk_get_default_root_window ();
+#endif
   
   attributes.x = 0;
   attributes.y = 0;
@@ -552,8 +627,8 @@ bonobo_dock_item_realize (GtkWidget *widget)
 			   GDK_FOCUS_CHANGE_MASK |
 			   GDK_STRUCTURE_MASK);
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
-  di->float_window = gdk_window_new (NULL, &attributes, attributes_mask);
-  gdk_window_set_transient_for(di->float_window, gdk_window_get_toplevel(widget->window));
+  di->float_window = gdk_window_new (root_window, &attributes, attributes_mask);
+  gdk_window_set_transient_for (di->float_window, gdk_window_get_toplevel (widget->window));
   gdk_window_set_user_data (di->float_window, widget);
   gdk_window_set_decorations (di->float_window, 0);
   
@@ -584,8 +659,7 @@ bonobo_dock_item_unrealize (GtkWidget *widget)
   gdk_window_destroy (di->float_window);
   di->float_window = NULL;
 
-  if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-    (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
+  GNOME_CALL_PARENT (GTK_WIDGET_CLASS, unrealize, (widget));
 }
 
 static void
@@ -667,7 +741,7 @@ bonobo_dock_item_size_request (GtkWidget      *widget,
 
 static void
 bonobo_dock_item_size_allocate (GtkWidget     *widget,
-                               GtkAllocation *allocation)
+				GtkAllocation *allocation)
 {
   GtkBin *bin;
   BonoboDockItem *di;
@@ -702,10 +776,21 @@ bonobo_dock_item_size_allocate (GtkWidget     *widget,
 
       if (BONOBO_DOCK_ITEM_NOT_LOCKED(di))
         {
-          if (di->orientation == GTK_ORIENTATION_HORIZONTAL)
+          GtkAllocation grip_alloc = *allocation;
+
+	  grip_alloc.x = grip_alloc.y = 0;
+
+          if (di->orientation == GTK_ORIENTATION_HORIZONTAL) {
             child_allocation.x += DRAG_HANDLE_SIZE;
-          else
+
+            grip_alloc.width = DRAG_HANDLE_SIZE;
+          } else {
             child_allocation.y += DRAG_HANDLE_SIZE;
+
+            grip_alloc.height = DRAG_HANDLE_SIZE;
+	  }
+
+	  gtk_widget_size_allocate (di->_priv->grip, &grip_alloc);
         }
 
       if (di->is_floating)
@@ -764,26 +849,15 @@ bonobo_dock_item_size_allocate (GtkWidget     *widget,
     }
 }
 
-static void
-draw_textured_frame (GtkWidget *widget, GdkWindow *window, GdkRectangle *rect, GtkShadowType shadow,
-		     GdkRectangle *clip)
-{
-  gtk_paint_handle(widget->style, window, GTK_STATE_NORMAL, shadow,
-                   clip, widget, "dockitem",
-                   rect->x, rect->y, rect->width, rect->height, 
-                   GTK_ORIENTATION_VERTICAL);
-}
-
 static void
 bonobo_dock_item_paint (GtkWidget      *widget,
-                       GdkEventExpose *event)
+			GdkEventExpose *event)
 {
   GtkBin *bin;
   BonoboDockItem *di;
   guint width;
   guint height;
   guint border_width;
-  GdkRectangle rect;
   gint drag_handle_size = DRAG_HANDLE_SIZE;
 
   if (!BONOBO_DOCK_ITEM_NOT_LOCKED (widget))
@@ -827,40 +901,14 @@ bonobo_dock_item_paint (GtkWidget      *widget,
                   "dockitem_bin",
                   0, 0, -1, -1);
 
-  /* We currently draw the handle _above_ the relief of the dockitem.
-     It could also be drawn on the same level...  */
-
   if (BONOBO_DOCK_ITEM_NOT_LOCKED (di))
-    {
-      GdkRectangle dest;
-
-      rect.x = 0;
-      rect.y = 0;
-      
-      if (di->orientation == GTK_ORIENTATION_HORIZONTAL)
-        {
-          rect.width = DRAG_HANDLE_SIZE;
-          rect.height = height;
-        }
-      else
-        {
-          rect.width = width;
-          rect.height = DRAG_HANDLE_SIZE;
-        }
-
-      dest = rect;
-
-      if (event == NULL ||
-	  gdk_rectangle_intersect (&event->area, &rect, &dest))
-	draw_textured_frame (widget, di->bin_window, &rect,
-			     GTK_SHADOW_OUT,
-			     &dest);
-    }    
+      gtk_container_propagate_expose (
+	      GTK_CONTAINER (di), di->_priv->grip, event);
 }
 
 static gint
 bonobo_dock_item_expose (GtkWidget      *widget,
-                        GdkEventExpose *event)
+			 GdkEventExpose *event)
 {
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (BONOBO_IS_DOCK_ITEM (widget), FALSE);
@@ -870,13 +918,23 @@ bonobo_dock_item_expose (GtkWidget      *widget,
     {
       bonobo_dock_item_paint (widget, event);
 
-      return GTK_WIDGET_CLASS (parent_class)->expose_event ?
-	  GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event) : FALSE;
+      if (GTK_WIDGET_CLASS (parent_class)->expose_event)
+	      return GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
     }
 
   return FALSE;
 }
 
+static void
+bonobo_dock_item_drag_end (BonoboDockItem *di)
+{
+  gdk_pointer_ungrab (GDK_CURRENT_TIME);
+
+  di->in_drag = FALSE;
+
+  g_signal_emit (di, dock_item_signals [DOCK_DRAG_END], 0);
+}
+
 static gint
 bonobo_dock_item_button_changed (GtkWidget      *widget,
                                 GdkEventButton *event)
@@ -929,24 +987,16 @@ bonobo_dock_item_button_changed (GtkWidget      *widget,
 	  di->dragoff_x = event->x;
 	  di->dragoff_y = event->y;
 
-	  di->in_drag = TRUE;
-
           bonobo_dock_item_grab_pointer (di);
 
-          gtk_signal_emit (GTK_OBJECT (widget),
-                           dock_item_signals[DOCK_DRAG_BEGIN]);
+          g_signal_emit (widget, dock_item_signals[DOCK_DRAG_BEGIN], 0);
 
 	  event_handled = TRUE;
 	}
     }
   else if (event->type == GDK_BUTTON_RELEASE && di->in_drag)
     {
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
-      
-      di->in_drag = FALSE;
-
-      gtk_signal_emit (GTK_OBJECT (widget),
-                       dock_item_signals[DOCK_DRAG_END]);
+      bonobo_dock_item_drag_end (di);
       event_handled = TRUE;
     }
 
@@ -955,9 +1005,10 @@ bonobo_dock_item_button_changed (GtkWidget      *widget,
 
 static gint
 bonobo_dock_item_motion (GtkWidget      *widget,
-                        GdkEventMotion *event)
+			 GdkEventMotion *event)
 {
   BonoboDockItem *di;
+  GdkWindow *root_window;
   gint new_x, new_y;
 
   g_return_val_if_fail (widget != NULL, FALSE);
@@ -971,14 +1022,20 @@ bonobo_dock_item_motion (GtkWidget      *widget,
   if (event->window != di->bin_window)
     return FALSE;
 
-  gdk_window_get_pointer (NULL, &new_x, &new_y, NULL);
+#ifdef HAVE_GTK_MULTIHEAD
+  root_window = gdk_screen_get_root_window (
+			gdk_drawable_get_screen (GDK_DRAWABLE (event->window)));
+#else
+  root_window = gdk_get_default_root_window ();
+#endif
+
+  gdk_window_get_pointer (root_window, &new_x, &new_y, NULL);
   
   new_x -= di->dragoff_x;
   new_y -= di->dragoff_y;
 
-  gtk_signal_emit (GTK_OBJECT (widget),
-                   dock_item_signals[DOCK_DRAG_MOTION],
-                   new_x, new_y);
+  g_signal_emit (widget, dock_item_signals[DOCK_DRAG_MOTION], 0,
+		 new_x, new_y);
 
   return TRUE;
 }
@@ -997,7 +1054,7 @@ bonobo_dock_item_add (GtkContainer *container,
   dock_item = BONOBO_DOCK_ITEM (container);
 
   gtk_widget_set_parent_window (widget, dock_item->bin_window);
-  GTK_CONTAINER_CLASS (parent_class)->add (container, widget);
+  GNOME_CALL_PARENT (GTK_CONTAINER_CLASS, add, (container, widget));
 
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (widget),
 					"orientation");
@@ -1032,15 +1089,29 @@ bonobo_dock_item_set_floating (BonoboDockItem *item, gboolean val)
 
 static void
 bonobo_dock_item_remove (GtkContainer *container,
-                        GtkWidget    *widget)
+			 GtkWidget    *widget)
 {
   BonoboDockItem *di;
 
   g_return_if_fail (BONOBO_IS_DOCK_ITEM (container));
-  g_return_if_fail (GTK_BIN (container)->child == widget);
 
   di = BONOBO_DOCK_ITEM (container);
 
+  if (widget == di->_priv->grip) {
+	  gboolean grip_was_visible;
+
+	  grip_was_visible = GTK_WIDGET_VISIBLE (widget);
+
+	  gtk_widget_unparent (widget);
+	  di->_priv->grip = NULL;
+
+	  if (grip_was_visible)
+		  gtk_widget_queue_resize (GTK_WIDGET (di));
+	  return;
+  }
+
+  g_return_if_fail (GTK_BIN (container)->child == widget);
+
   if (di->is_floating)
     {
       bonobo_dock_item_set_floating (di, FALSE);
@@ -1053,12 +1124,28 @@ bonobo_dock_item_remove (GtkContainer *container,
       di->float_window_mapped = FALSE;
     }
   if (di->in_drag)
-    {
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
-      di->in_drag = FALSE;
-    }
-  
-  GTK_CONTAINER_CLASS (parent_class)->remove (container, widget);
+    bonobo_dock_item_drag_end (di);
+
+  GNOME_CALL_PARENT (GTK_CONTAINER_CLASS,
+		     remove, (container, widget));
+}
+
+static void
+bonobo_dock_item_forall (GtkContainer *container,
+			 gboolean      include_internals,
+			 GtkCallback   callback,
+			 gpointer      callback_data)
+{
+  GtkBin *bin = (GtkBin *) container;
+  BonoboDockItem *di = (BonoboDockItem *) container;
+
+  g_return_if_fail (callback != NULL);
+
+  if (di->_priv->grip)
+    callback (di->_priv->grip, callback_data);
+
+  if (bin->child)
+    callback (bin->child, callback_data);
 }
 
 static gint
@@ -1072,8 +1159,6 @@ bonobo_dock_item_delete_event (GtkWidget *widget,
   return TRUE;
 }
 
-
-
 /**
  * bonobo_dock_item_construct:
  * @new: a #BonoboDockItem.
@@ -1090,11 +1175,17 @@ bonobo_dock_item_construct (BonoboDockItem *new,
 			   const gchar *name,
 			   BonoboDockItemBehavior behavior)
 {
-	g_return_if_fail (new != NULL);
-	g_return_if_fail (BONOBO_IS_DOCK_ITEM (new));
+  g_return_if_fail (new != NULL);
+  g_return_if_fail (BONOBO_IS_DOCK_ITEM (new));
 	
-	new->name = g_strdup (name);
-	new->behavior = behavior;
+  new->name = g_strdup (name);
+  new->behavior = behavior;
+
+  if (behavior & BONOBO_DOCK_ITEM_BEH_LOCKED)
+    {
+      gtk_widget_hide (new->_priv->grip);
+      GTK_WIDGET_UNSET_FLAGS (new->_priv->grip, GTK_CAN_FOCUS);
+    }
 }
 
 /**
@@ -1107,13 +1198,13 @@ bonobo_dock_item_construct (BonoboDockItem *new,
  * 
  * Returns: A new BonoboDockItem widget.
  **/
-GtkWidget*
+GtkWidget *
 bonobo_dock_item_new (const gchar *name,
-                     BonoboDockItemBehavior behavior)
+		      BonoboDockItemBehavior behavior)
 {
   BonoboDockItem *new;
 
-  new = BONOBO_DOCK_ITEM (gtk_type_new (bonobo_dock_item_get_type ()));
+  new = BONOBO_DOCK_ITEM (g_object_new (bonobo_dock_item_get_type (), NULL));
 
   bonobo_dock_item_construct (new, name, behavior);
 
@@ -1157,10 +1248,9 @@ bonobo_dock_item_get_name (BonoboDockItem *item)
  * Description: Set the shadow type for @dock_item.
  **/
 void
-bonobo_dock_item_set_shadow_type (BonoboDockItem  *dock_item,
-                                 GtkShadowType  type)
+bonobo_dock_item_set_shadow_type (BonoboDockItem *dock_item,
+				  GtkShadowType   type)
 {
-  g_return_if_fail (dock_item != NULL);
   g_return_if_fail (BONOBO_IS_DOCK_ITEM (dock_item));
 
   if (dock_item->shadow_type != type)
@@ -1168,7 +1258,7 @@ bonobo_dock_item_set_shadow_type (BonoboDockItem  *dock_item,
       dock_item->shadow_type = type;
 
       if (GTK_WIDGET_DRAWABLE (dock_item))
-        gtk_widget_queue_clear (GTK_WIDGET (dock_item));
+        gtk_widget_queue_draw (GTK_WIDGET (dock_item));
       gtk_widget_queue_resize (GTK_WIDGET (dock_item));
     }
 }
@@ -1201,7 +1291,7 @@ bonobo_dock_item_get_shadow_type (BonoboDockItem  *dock_item)
  **/
 gboolean
 bonobo_dock_item_set_orientation (BonoboDockItem *dock_item,
-                                 GtkOrientation orientation)
+				  GtkOrientation  orientation)
 {
   g_return_val_if_fail (dock_item != NULL, FALSE);
   g_return_val_if_fail (BONOBO_IS_DOCK_ITEM (dock_item), FALSE);
@@ -1226,10 +1316,10 @@ bonobo_dock_item_set_orientation (BonoboDockItem *dock_item,
 	g_value_unset (&value);
       }
       if (GTK_WIDGET_DRAWABLE (dock_item))
-        gtk_widget_queue_clear (GTK_WIDGET (dock_item));
+        gtk_widget_queue_draw (GTK_WIDGET (dock_item));
       gtk_widget_queue_resize (GTK_WIDGET (dock_item));
 
-      gtk_signal_emit(GTK_OBJECT(dock_item), dock_item_signals[ORIENTATION_CHANGED], orientation);
+      g_signal_emit (dock_item, dock_item_signals[ORIENTATION_CHANGED], 0, orientation);
     }
 
   return TRUE;
@@ -1246,14 +1336,48 @@ bonobo_dock_item_set_orientation (BonoboDockItem *dock_item,
 GtkOrientation
 bonobo_dock_item_get_orientation (BonoboDockItem *dock_item)
 {
-  g_return_val_if_fail (dock_item != NULL,
-                        GTK_ORIENTATION_HORIZONTAL);
   g_return_val_if_fail (BONOBO_IS_DOCK_ITEM (dock_item),
                         GTK_ORIENTATION_HORIZONTAL);
 
   return dock_item->orientation;
 }
 
+/**
+ * bonobo_dock_item_set_behavior:
+ * @dock_item: A BonoboDockItem widget.
+ * @behavior: New behavior for @dock_item
+ *
+ * Description: Set the behavior for @dock_item.
+ */
+void
+bonobo_dock_item_set_behavior (BonoboDockItem         *dock_item,
+                               BonoboDockItemBehavior  behavior)
+{
+  g_return_if_fail (BONOBO_IS_DOCK_ITEM (dock_item));
+
+  if (dock_item->behavior == behavior)
+    return;
+
+  dock_item->behavior = behavior;
+
+  if (behavior & BONOBO_DOCK_ITEM_BEH_LOCKED)
+    bonobo_dock_item_set_locked (dock_item, TRUE);
+
+  if (behavior & BONOBO_DOCK_ITEM_BEH_NEVER_FLOATING &&
+      dock_item->is_floating)
+    bonobo_dock_item_unfloat (dock_item);
+
+  if (behavior & BONOBO_DOCK_ITEM_BEH_NEVER_VERTICAL &&
+      dock_item->orientation == GTK_ORIENTATION_VERTICAL)
+    bonobo_dock_item_set_orientation (dock_item, GTK_ORIENTATION_HORIZONTAL);
+
+  if (behavior & BONOBO_DOCK_ITEM_BEH_NEVER_HORIZONTAL &&
+      dock_item->orientation == GTK_ORIENTATION_HORIZONTAL)
+    bonobo_dock_item_set_orientation (dock_item, GTK_ORIENTATION_VERTICAL);
+
+  gtk_widget_queue_resize (GTK_WIDGET (dock_item));
+}
+
 /**
  * bonobo_dock_item_get_behavior:
  * @dock_item: A BonoboDockItem widget.
@@ -1265,23 +1389,47 @@ bonobo_dock_item_get_orientation (BonoboDockItem *dock_item)
 BonoboDockItemBehavior
 bonobo_dock_item_get_behavior (BonoboDockItem *dock_item)
 {
-  g_return_val_if_fail (dock_item != NULL,
-                        BONOBO_DOCK_ITEM_BEH_NORMAL);
   g_return_val_if_fail (BONOBO_IS_DOCK_ITEM (dock_item),
                         BONOBO_DOCK_ITEM_BEH_NORMAL);
 
   return dock_item->behavior;
 }
 
-
-
 /* Private interface.  */
 
+void
+bonobo_dock_item_set_locked (BonoboDockItem *dock_item,
+			     gboolean        locked)
+{
+  g_return_if_fail (BONOBO_IS_DOCK_ITEM (dock_item));
+
+  if (locked)
+    {
+      if (!BONOBO_DOCK_ITEM_NOT_LOCKED (dock_item))
+        return;
+
+      dock_item->behavior |= BONOBO_DOCK_ITEM_BEH_LOCKED;
+      gtk_widget_hide (dock_item->_priv->grip);
+    }
+  else
+    {
+      if (BONOBO_DOCK_ITEM_NOT_LOCKED (dock_item))
+        return;
+
+      dock_item->behavior &= ~BONOBO_DOCK_ITEM_BEH_LOCKED;
+      gtk_widget_show (dock_item->_priv->grip);
+    }
+}
+
 void
 bonobo_dock_item_grab_pointer (BonoboDockItem *item)
 {
   GdkCursor *fleur;
 
+  g_assert (BONOBO_IS_DOCK_ITEM (item));
+
+  item->in_drag = TRUE;
+
   fleur = gdk_cursor_new (GDK_FLEUR);
 
   /* Hm, not sure this is the right thing to do, but it seems to work.  */
@@ -1294,14 +1442,14 @@ bonobo_dock_item_grab_pointer (BonoboDockItem *item)
                            fleur,
                            GDK_CURRENT_TIME) != 0);
 
-  gdk_cursor_destroy (fleur);
+  gdk_cursor_unref (fleur);
 }
 
 gboolean
 bonobo_dock_item_detach (BonoboDockItem *item, gint x, gint y)
 {
-  GtkRequisition requisition;
   GtkAllocation allocation;
+  GtkRequisition requisition;
 
   if (item->behavior & BONOBO_DOCK_ITEM_BEH_NEVER_FLOATING)
     return FALSE;
@@ -1320,7 +1468,8 @@ bonobo_dock_item_detach (BonoboDockItem *item, gint x, gint y)
                           x, y, requisition.width, requisition.height);
 
   gdk_window_reparent (item->bin_window, item->float_window, 0, 0);
-  gdk_window_set_hints (item->float_window, x, y, 0, 0, 0, 0, GDK_HINT_POS);
+	  
+/*  gdk_window_set_hints (item->float_window, x, y, 0, 0, 0, 0, GDK_HINT_POS); */
 
   gdk_window_show (item->float_window);
   item->float_window_mapped = TRUE;
@@ -1340,23 +1489,31 @@ bonobo_dock_item_detach (BonoboDockItem *item, gint x, gint y)
 }
 
 void
-bonobo_dock_item_attach (BonoboDockItem *item, GtkWidget *parent, gint x, gint y)
+bonobo_dock_item_unfloat (BonoboDockItem *item)
+{
+  gdk_window_move_resize (GTK_WIDGET (item)->window, -1, -1, 0, 0);
+
+  gdk_window_hide (item->float_window);
+
+  gdk_window_reparent (item->bin_window, GTK_WIDGET (item)->window, 0, 0);
+  gdk_window_show (GTK_WIDGET (item)->window);
+  
+  item->float_window_mapped = FALSE;
+  bonobo_dock_item_set_floating (item, FALSE);
+  
+  gtk_widget_queue_resize (GTK_WIDGET (item));
+}
+
+void
+bonobo_dock_item_attach (BonoboDockItem *item,
+			 GtkWidget *parent,
+			 gint x, gint y)
 {
   if (GTK_WIDGET (item)->parent != GTK_WIDGET (parent))
     {
       gdk_window_move_resize (GTK_WIDGET (item)->window, -1, -1, 0, 0);
       gtk_widget_reparent (GTK_WIDGET (item), parent);
-
-      gdk_window_hide (item->float_window);
-
-      gdk_window_reparent (item->bin_window, GTK_WIDGET (item)->window, 0, 0);
-      gdk_window_show (GTK_WIDGET (item)->window);
-
-      item->float_window_mapped = FALSE;
-      bonobo_dock_item_set_floating (item, FALSE);
-
-      gtk_widget_queue_resize (GTK_WIDGET (item));
-
+      bonobo_dock_item_unfloat (item);
       bonobo_dock_item_grab_pointer (item);
     }
 }
@@ -1398,7 +1555,7 @@ bonobo_dock_item_handle_size_request (BonoboDockItem *item,
 
 void
 bonobo_dock_item_get_floating_position (BonoboDockItem *item,
-                                       gint *x, gint *y)
+					gint *x, gint *y)
 {
   if (GTK_WIDGET_REALIZED (item) && item->is_floating)
     gdk_window_get_position (item->float_window, x, y);
@@ -1408,3 +1565,14 @@ bonobo_dock_item_get_floating_position (BonoboDockItem *item,
       *y = item->float_y;
     }
 }
+
+GtkWidget *
+bonobo_dock_item_get_grip (BonoboDockItem *item)
+{
+  g_return_val_if_fail (BONOBO_IS_DOCK_ITEM (item), NULL);
+
+  if (item->behavior & BONOBO_DOCK_ITEM_BEH_LOCKED)
+    return NULL;
+  else
+    return item->_priv->grip;
+}
